"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const glob = require("glob");
const MaxAPIFacade_1 = require("./MaxAPIFacade");
const DEFAULT_CONFIG = require('./constants/default-config');
// Do not export singleton class
const ConfigLoader = () => {
    const resolvePath = (config, patcherDir) => {
        const ret = Object.assign({}, config);
        if (config.testCodeDir) {
            ret.testCodeDir = path.resolve(patcherDir || '', config.testCodeDir);
        }
        return ret;
    };
    const load = (fileName) => {
        switch (path.extname(fileName)) {
            case '.js':
            case '.json':
                return require(fileName);
            default:
                throw new Error('invalid config file type');
        }
    };
    return {
        run() {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const patcherDir = yield MaxAPIFacade_1.maxAPIFacade.patcherDir();
                    const configFiles = glob.sync(path.join(patcherDir, '.tddmaxrc.*'));
                    const inputConfig = configFiles.length ? load(configFiles[0]) : {};
                    return Object.assign({}, DEFAULT_CONFIG, {
                        testCodeDir: patcherDir
                    }, resolvePath(inputConfig, patcherDir));
                }
                catch (e) {
                    return DEFAULT_CONFIG;
                }
            });
        }
    };
};
exports.configLoader = ConfigLoader();
